@@ -3,7 +3,7 @@
 /*
     Rubber Band
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2009 Chris Cannam.
+    Copyright 2007-2010 Chris Cannam.
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -21,11 +21,11 @@
 
 //#define EXPERIMENT 1
 
-namespace RubberBand { class Resampler; }
-	
 namespace RubberBand
 {
 
+class Resampler;
+
 class RubberBandStretcher::Impl::ChannelData
 {
 public:        
