@@ -3,7 +3,7 @@
 /*
     Rubber Band
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2009 Chris Cannam.
+    Copyright 2007-2010 Chris Cannam.
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -23,13 +23,11 @@ namespace RubberBand
 class ConstantAudioCurve : public AudioCurveCalculator
 {
 public:
-    ConstantAudioCurve(size_t sampleRate, size_t windowSize);
+    ConstantAudioCurve(Parameters parameters);
     virtual ~ConstantAudioCurve();
 
-    virtual void setWindowSize(size_t newSize);
-
-    virtual float processFloat(const float *R__ mag, size_t increment);
-    virtual double processDouble(const double *R__ mag, size_t increment);
+    virtual float processFloat(const float *R__ mag, int increment);
+    virtual double processDouble(const double *R__ mag, int increment);
     virtual void reset();
 };
 
