@@ -3,7 +3,7 @@
 /*
     Rubber Band
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2009 Chris Cannam.
+    Copyright 2007-2010 Chris Cannam.
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -26,6 +26,8 @@
 #endif
 
 
+
+
 #ifdef USE_KISSFFT
 #include "kissfft/kiss_fftr.h"
 #endif
@@ -80,6 +82,8 @@ namespace FFTs {
 
 
 
+
+
 #ifdef HAVE_FFTW3
 
 // Define FFTW_DOUBLE_ONLY to make all uses of FFTW functions be
@@ -247,7 +251,7 @@ public:
         if (!home) return;
 
         char fn[256];
-        snprintf(fn, 256, "%s/%s.%c", home, ".turbot.wisdom", type);
+        snprintf(fn, 256, "%s/%s.%c", home, ".rubberband.wisdom", type);
 
         FILE *f = fopen(fn, save ? "wb" : "rb");
         if (!f) return;
@@ -1280,6 +1284,26 @@ FFT::FFT(int size, int debugLevel)
 #endif
         break;
 
+    case 5:
+        std::cerr << "FFT::FFT(" << size << "): WARNING: Selected implemention not available" << std::endl;
+#ifdef USE_BUILTIN_FFT
+        d = new FFTs::D_Cross(size);
+#else
+        std::cerr << "FFT::FFT(" << size << "): ERROR: Fallback implementation not available!" << std::endl;
+        abort();
+#endif
+        break;
+
+    case 6:
+        std::cerr << "FFT::FFT(" << size << "): WARNING: Selected implemention not available" << std::endl;
+#ifdef USE_BUILTIN_FFT
+        d = new FFTs::D_Cross(size);
+#else
+        std::cerr << "FFT::FFT(" << size << "): ERROR: Fallback implementation not available!" << std::endl;
+        abort();
+#endif
+        break;
+
     default:
 #ifdef USE_BUILTIN_FFT
         std::cerr << "FFT::FFT(" << size << "): WARNING: using slow built-in implementation" << std::endl;
