@@ -3,7 +3,7 @@
 /*
     Rubber Band
     An audio time-stretching and pitch-shifting library.
-    Copyright 2007-2009 Chris Cannam.
+    Copyright 2007-2010 Chris Cannam.
     
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License as
@@ -24,15 +24,16 @@ namespace RubberBand
 class SpectralDifferenceAudioCurve : public AudioCurveCalculator
 {
 public:
-    SpectralDifferenceAudioCurve(size_t sampleRate, size_t windowSize);
+    SpectralDifferenceAudioCurve(Parameters parameters);
 
     virtual ~SpectralDifferenceAudioCurve();
 
-    virtual void setWindowSize(size_t newSize);
+    virtual void setWindowSize(int newSize);
 
-    virtual float processFloat(const float *R__ mag, size_t increment);
-    virtual double processDouble(const double *R__ mag, size_t increment);
+    virtual float processFloat(const float *R__ mag, int increment);
+    virtual double processDouble(const double *R__ mag, int increment);
     virtual void reset();
+    virtual const char *getUnit() const { return "V"; }
 
 protected:
     double *R__ m_mag;
